﻿
Imports System.Data.Entity
Imports System.Linq.Expressions
Imports UoWTestApp.Core.Repositories
Imports UoWTestApp.Interfaces

Namespace Persistence.Repositories
    Public Class Repository(Of TEntity As Class)
        Implements IRepository(Of TEntity)

        Protected ReadOnly Context As DbContext

        Public Sub New(ByVal _context As DbContext)
            Context = _context
        End Sub

        Public Function [Get](ByVal id As Guid) As TEntity Implements IRepository(Of TEntity).Get
            Return Context.[Set](Of TEntity)().Find(id)
        End Function

        Public Function GetAll(Optional IncludeDeleted As Boolean = False) As IQueryable(Of TEntity) Implements IRepository(Of TEntity).GetAll
            Dim Query As IQueryable(Of TEntity) = Context.Set(Of TEntity)
            If Not IncludeDeleted Then
                If GetType(IDelete).IsAssignableFrom(GetType(TEntity)) Then
                    Query = Query.Where(Function(x) DirectCast(x, IDelete).Deleted = False)
                End If
            End If
            Return Query
        End Function

        Public Function Find(ByVal predicate As Expression(Of Func(Of TEntity, Boolean))) As IEnumerable(Of TEntity) Implements IRepository(Of TEntity).Find
            Return Context.[Set](Of TEntity)().Where(predicate)
        End Function

        Public Function SingleOrDefault(ByVal predicate As Expression(Of Func(Of TEntity, Boolean))) As TEntity Implements IRepository(Of TEntity).SingleOrDefault
            Return Context.[Set](Of TEntity)().SingleOrDefault(predicate)
        End Function

        Public Sub Add(ByVal entity As TEntity) Implements IRepository(Of TEntity).Add
            Context.[Set](Of TEntity)().Add(entity)
        End Sub

        Public Sub AddRange(ByVal entities As IEnumerable(Of TEntity)) Implements IRepository(Of TEntity).AddRange
            Context.[Set](Of TEntity)().AddRange(entities)
        End Sub

        Public Sub Remove(ByVal entity As TEntity) Implements IRepository(Of TEntity).Remove
            Context.[Set](Of TEntity)().Remove(entity)
        End Sub

        Public Sub RemoveRange(ByVal entities As IEnumerable(Of TEntity)) Implements IRepository(Of TEntity).RemoveRange
            Context.[Set](Of TEntity)().RemoveRange(entities)
        End Sub


    End Class
End Namespace